@@ -26,7 +26,7 @@
 ##  shiela-install.sh: repository install program (syntax: Bourne-Shell)
 ##
 
-version="1.0.3"
+version="1.0.4"
 
 prefix="@prefix@"
 bindir="@bindir@"
@@ -257,6 +257,11 @@ if [ ".$V_repos_history" = . ]; then
 fi
 query U repos_history
 
+if [ ".$V_env_path" = . ]; then
+    V_env_path="/bin:/usr/bin:/sbin:/usr/sbin"
+fi
+query I env_path
+
 #   check whether the test command supports the -x option
 if [ -x /bin/sh ] 2>/dev/null; then
     minusx="-x"
@@ -307,7 +312,7 @@ if [ ".$V_tool_sendmail" = . ]; then
         fi
     done
 fi
-query U tool_sendmail
+query I tool_sendmail
 
 if [ ".$V_tool_diff" = . ]; then
     V_tool_diff="diff"
@@ -319,7 +324,7 @@ if [ ".$V_tool_diff" = . ]; then
         fi
     done
 fi
-query U tool_diff
+query I tool_diff
 
 if [ ".$V_tool_xdelta" = . ]; then
     V_tool_xdelta="xdelta"
@@ -331,7 +336,7 @@ if [ ".$V_tool_xdelta" = . ]; then
         fi
     done
 fi
-query U tool_xdelta
+query I tool_xdelta
 
 if [ ".$V_tool_uuencode" = . ]; then
     V_tool_uuencode="uuencode"
@@ -343,7 +348,7 @@ if [ ".$V_tool_uuencode" = . ]; then
         fi
     done
 fi
-query U tool_uuencode
+query I tool_uuencode
 
 if [ ".$V_tool_cvs" = . ]; then
     V_tool_cvs="cvs"
@@ -432,6 +437,7 @@ cat >$tmpdir/shiela.cfg <<EOT
 ##
 
 Environment {
+    Setenv  PATH     ${V_env_path};
     Program sendmail ${V_tool_sendmail};
     Program cvs      ${V_tool_cvs};
     Program diff     ${V_tool_diff};
